#!/usr/local/bin/perl

$mailprog = '/bin/sendmail';


#   $Id: directpaycredit.cgi,v 1.12 1998/04/02 16:59:29 grass Exp $
#
#   System     :   Merchant Connection Kit (MCK)
#
#   Author     :   CYCH Development staff
#   Description:   CGI script to do direct submission of a payment
#   Notes      :   This may be edited to integrate with the merchant's
#                   storefront
#
#                         COPYRIGHT NOTICE
#
#   The contents of this file is protected under the United States
#   copyright laws as an unpublished work, and is confidential and
#   proprietary to CyberCash, Inc.  Its use or disclosure in whole or in
#   part without the expressed written permission of CyberCash, Inc. is
#   prohibited.
#
#   (c) Copyright 1993-98 by CyberCash, Inc.  All rights reserved.
#


###########################
#
# directpaycredit.cgi
#
#    RCSID $Id: directpaycredit.cgi,v 1.12 1998/04/02 16:59:29 grass Exp $
#
###########################


    # 
    # Check Perl version
    #
require 5.003;

    #
    # Require some libraries:  either install these where PERL can find them,
    #   or add code to push the directory where they live...
    #

# The following 4 lines are here only because the current directory for
# Microsoft IIS when running script on browser is always the root
# directory of the web server.
#BEGIN {
#     push(@INC, '/www/htdocs/encrypt/test-mck/mck-cgi');
#     push(@INC, '/www/htdocs/encrypt/test-mck/mck-cgi/');
#}


use CCMckLib3_2 qw($MCKversion  %Config URLdecodeForm
	InitConfig CCError CCDebug CCDebug2 GetQuery
	PrintTemplate LoadBlockFromQuery
	BuildSignature );

use CCMckDirectLib3_2 qw(doDirectPayment);

use CCMckErrno3_2 qw(MCKGetErrorMessage $E_NoErr
    $E_Fail_Notif
    $E_Payment_Failed
    $E_No_Receipt
	$E_MO_Signature
	$E_MF_Signature
	$E_No_Template
    );

use CCMerchantTest qw($ConfigFile TemplateFailure LogNotification);

use CCMerchantCustom qw( LoadPaymentInfo SendPayload);


##################
#  
#  Some data initializations....
#
##################


# Initialize the Global Configuration array in CCMckLib3_2
($status) = &InitConfig($ConfigFile);

if ($status != 0) {
    $errmsg = &MCKGetErrorMessage($status);
    &TemplateFailure($errmsg);
    die("$0: $errmsg.\n");
}

&CCDebug("Entering $0\n");

$cybercashId = $Config{"CYBERCASH_ID"};
$hashSecret = $Config{"HASH_SECRET"};
$ccpsHost = $Config{"CCPS_HOST"};
$templateDir = $Config{"TEMPLATE_DIR"};
$redirectURL = "shopfactoryemail.cgi";			# $Config{"REDIRECT_URL"};

	# A Note about message fields in paymentNVList.
	#    paymentNVList holds message parameters 
	#    that will be passed to the Cash Register
	#  
	# These parameters are broken into three blocks according to
	# a prefix in the argument field name.
	#
	# prefix                 use
	#
	#  mo.   -- data driving the Cash Register 
	#                 this include Payment data and some operational options
	#  cpi.  -- info about the Customer's "Payment Instrument: (credit/check)
	#  mf.   -- anything additional you want to pass on 
    #           to the Fulfillment Center

$paymentNVList{'mo.cybercash-id'} = $cybercashId;
$paymentNVList{'mo.version'} = $MCKversion;  #CCPS message version

	# The MO and CPI blocks are not signed, since 
	# the message will be encrypted

$paymentNVList{'mo.signed-cpi'} = "no";

$problemTemplate =  $templateDir . "tempDifficulties.tem";


	#   Once the payment results are in, you can generate a receipt or
	#   dispense a payload immediately within this CGI.
	#   
	#   *OR*, you can redirect the response to a remote Fulfillment center
	#   and allow that Fulfillment Center to generate a receipt or dispense
	#   the payload.
	#
	#   Indicate a redirection by including the field:
	#     "mo.redirect-url" 
	#   in your request.
	#
	#  If you redirect, customRedirectResponse.tem
    #  template will be used to generate a redirection page.
	#
	#  You may use customReceipt.tem for a locally generated receipt
	#  and customFailureResponse.tem if the payment fails somehow.
	#
	#   you may customize these pages as you wish.

	# If you are passing anything in the MF section of the message
	# to a fulfillment center via a Redirect,
	# You had better use an SSL connection (https://) in the redirectURL.

if ($redirectURL) {
	$paymentNVList{'mo.redirect-url'} = $redirectURL;
}

	# Set up the Payment Response templates.

if ($redirectURL) {
    $successTemplate = $templateDir . "customRedirectResponse.tem";
}
else {
    $successTemplate = $templateDir . "customReceipt.tem";
}
 
$failTemplate    = $templateDir . "customFailureResponse.tem";

#############################
#  MAIN code starts here    #
#############################

	# get the query string
%query = &GetQuery;


	# Some payment pages feed into this, so the MO may be present as a MO Block.
	# If there is an MO block, unpack it here...

if (defined ($query{'MO'})) {

	$MOstr = $query{'MO'};
	delete $query{'MO'};

	%MO = &URLdecodeForm($MOstr);

	# Note:  You, the merchant, may sign an MO block.
	# If you do, verify the signature here....
	
	if (defined($MO{'mo.sign'})) {   # signature test here... if you need to
		$sign = $MO{'mo.sign'};
		delete $MO{'mo.sign'};  ## don't sign the sign... and don't log the sign
		 
		$tmpSign = &BuildSignature("mo", $hashSecret, %MO);
		
		if ($tmpSign ne $sign) 
		{
			#GONG!
			$errmsg = &MCKGetErrorMessage($E_MO_Signature);
			$templatePrms{'MESSAGE'} = $errmsg;
			$templatePrms{'ORDERID'} = $MO{'mo.order-id'};
		
			&CCError("$0: $errmsg.\n");
			$errcode = &PrintTemplate($problemTemplate, %templatePrms);
			if ($errcode != $E_NoErr) {
				&TemplateFailure ($errmsg);
			}
			exit (1);
		}
	}

	# Load the MO stuff into the query vector....
	# So all of the individual mo fields will be in the query vector
	# Beyond this, it won't matter if the MO arrived as a block or
	# as unblocked elements.

	foreach $key (keys (%MO)) {
        $query{$key} = $MO{$key};
    }
}

    #Read the order-id from the message
$orderId = $query{'mo.order-id'};

	#Load Payment MO information from query into the payment args list
	#Key is the orderId
	# This function reports, logs then dies if there is a problem

%paymentNVList = &LoadPaymentInfo($orderId, \%query, \%paymentNVList, $problemTemplate); 


# load payment instrument info from form

# This is a CREDIT CARD payment
# DON'T TOUCH THIS!  This is the CR script that will process your payment

$script = "directcardpayment.cgi";
	
$paymentNVList{'cpi.card-number'} = $query{'cpi.card-number'};
$paymentNVList{'cpi.card-exp'} = $query{'cpi.card-exp'};
$paymentNVList{'cpi.card-name'} = $query{'cpi.card-name'};
$paymentNVList{'cpi.card-address'} = $query{'cpi.card-address'};
$paymentNVList{'cpi.card-city'} = $query{'cpi.card-city'};
$paymentNVList{'cpi.card-state'} = $query{'cpi.card-state'};
$paymentNVList{'cpi.card-zip'} = $query{'cpi.card-zip'};
$paymentNVList{'cpi.card-country'} = $query{'cpi.card-country'};


	###########################
	#  No signatures required to talk to the Cash Register
	#  NO SSL required to talk to the Cash Register either...
    #  Since the whole thing is encrypted, crooks can't mess with it
    #  and can't read it either!
	###########################

# If a payment page is driving this, you may have a signed MF blob here...
# Unpack it and check the signature....

if (defined ($query{'MF'})) {

	$MFstr = $query{'MF'};
	delete $query{'MF'};

	%MF = &URLdecodeForm($MFstr);

	# Note:  You, the merchant, may sign such blocks.
	# If you do, verify the signature here....
	
	if (defined($MF{'mf.sign'})) { 
		$sign = $MF{'mf.sign'};
		delete $MF{'mf.sign'};  ## don't sign the sign... and don't log the sign
		 
		$tmpSign = &BuildSignature("mf", $hashSecret, %MF);
		
		if ($tmpSign ne $sign) 
		{
			#GONG!
			$errmsg = &MCKGetErrorMessage($E_MF_Signature);
			$templatePrms{'MESSAGE'} = $errmsg;
			$templatePrms{'ORDERID'} = $orderId;
		
			&CCError("$0: $errmsg.\n");
			$errcode = &PrintTemplate($problemTemplate, %templatePrms);
			if ($errcode != $E_NoErr) {
				&TemplateFailure ($errmsg);
			}
			exit (1);
		}
	}

	# Load the MF stuff into the query vector....
	# Beyond this, it will not matter which MF items were in a block
	# and which ones were individual query entries.

	foreach $key (keys (%MF)) {
        $query{$key} = $MF{$key};
    }
}


#Load any MF fields that may be in the query
#  This will copy any query field with "mf." prefix into the payment vector

%paymentNVList = &LoadBlockFromQuery("mf", \%query, \%paymentNVList);

#add anything additional you like to this message, 
#  we'll pass it on to the FC!
# $paymentNVList{'mf.your-field-here'} = "some value";

# if you want to sign the MF, go ahead...
# That will allow you to check the signature at a fulfillment center.
# You may also provide some other kind of security for your MF data if
# you will be doing remote fulfillment.
# 
# If you are not redirecting the consumer to a fulfillment center, 
# there is no reason to sign the MF, since the data will not be exposed
#
# Adding a signature only makes sense if 
#	1) you are redirecting to a Fulfillment Center
#   2) you can check the signature at the FulFillment Center 

if ($redirectURL) {
	$paymentNVList{'mf.sign'} = &BuildSignature("mf", $hashSecret, %paymentNVList);
}

	#############################
	# Generate a Direct Payment #
	#############################


# merchant-id gets added to the URL  by the connection routine.
# Don't add it here...

$paymentURL = "$ccpsHost$script";


#At this point, all of the data that needs to go into the Payment message
# should have been collected: So, we can call the Cash Register

($POPref, $tokenListref) = &doDirectPayment( $paymentURL, \%paymentNVList);

%POP = %$POPref;
%tokenList = %$tokenListref;


#At this point, we do fulfillment....
#  Make absolutely everything available for template building
#
# If you have anything else you need to record, like
# shipping address, etc.  You may do that here...
# 

#Log Payment results here...
	
$POPstr = $tokenList{'POP'};
($logStatus, $logCode) = &LogNotification($POPstr);

# what you *DO* about a logging failure depends on a number of things..
# Always log a failure (if possible)
# For payload sales, we'll still dispnse the payload on a successful
# payment that failed to log.  For hard goods sales, we'll log the logging
# failure and have to satisfy the customer some other way.

if ($logStatus != $E_NoErr) {
	# Log the failure in any case...
	&CCError("$0: Order Id $orderId: "
		. &MCKGetErrorMessage($E_Fail_Notif) . " - "
		. &MCKGetErrorMessage($logStatus) ." \n"
	);
	&CCError("$0: Log Entry: $POPstr\n");

	if (!defined $POP{'pop.payload'}) {
		# If logging failed, hard goods won't be delivered
		# This is fatal

		die("$0: ". &MCKGetErrorMessage($E_Fail_Notif) . "\.n");
	}
}

# There are two possible ways to do fulfillment:
# 1) do it right here
# 2) redirect this response to a remote fulfillment location
# Let's start with remote  fulfillment....

if ($POP{'pop.status'} !~ /^success/) {

	# bombed out.. respond using the customFailureResponse page
	$errcode = &PrintTemplate($failTemplate, %tokenList);
	if ($errcode != $E_NoErr) {
		&TemplateFailure("$E_Payment_Failed");
	}
	
}
else {
	if($query{'mf.recipient'} eq "") {
		print "Content-type: text/html\n\n";
		print "<body";
		&body_attributes;
		print ">\n";
		print "<br><br><br><br><center>\n";
		print "<font size=\"+2\">The Company Details Wizard in ShopFactory requires an E-mail Address email to which the order should be sent.</font><br><br>\n";
		print "</center>";
		print "</body>\n</html>";
		die;
	}

	&get_date;

	open(MAIL,"|$mailprog -t");
	print MAIL "To: $query{'mf.recipient'}\n";
	print MAIL "From: $query{'mf.customer_name'} ($query{'mf.customer_email'})\n";
	print MAIL "Subject: Order From $query{'mf.shopname'}\n";
	print MAIL "\n----------------------------------------\n";
	print MAIL "Below is an order from $query{'mf.shopname'}\n";
	print MAIL "Submitted by: $query{'mf.customer_name'}\n";
	print MAIL "       email: $query{'mf.customer_email'}\n";
	print MAIL "        date: $date\n";
	print MAIL "----------------------------------------\n\n";

	print MAIL "Order ID: $query{'mo.order-id'}\n\n";
	print MAIL "$POP{'pop.product-descr'}\n";

	print MAIL "\nBrowser: $ENV{'HTTP_USER_AGENT'}\n";
	print MAIL "Remote Host: $ENV{'REMOTE_HOST'}\n";
	print MAIL "Remote Address: $ENV{'REMOTE_ADDR'}\n";

	close (MAIL);

	print "Content-type: text/html\n\n";

	print "<head>\n";
	print "<meta http-equiv=\"refresh\" content=\"3;url=$query{'mf.thankyou_url'}\">\n";
	print "</head>\n";
	print "<body bgcolor=\"#ffffff\" text=\"#000000\">\n";
	print "<br><br><br><br><center>\n";
	print "<font size=\"+2\">SSL Secured transaction finalized.</font><br><br>\n";
	print "<a href=\"$query{mf.'thankyou_url'}\"><font size=\"-1\">Next</font></a>\n";
	print "</center>";
	print "</body>\n</html>";

}

&CCDebug("Exiting $0\n");

exit 0;

sub get_date {
	@days = ('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday');
	@months = ('January','February','March','April','May','June','July','August','September','October','November','December');

	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	if ($hour < 10) { $hour = "0$hour"; }
	if ($min < 10) { $min = "0$min"; }
	if ($sec < 10) { $sec = "0$sec"; }

	$date = "$days[$wday], $months[$mon] $mday, $year at $hour\:$min\:$sec";
}

